@@ -1,7 +1,7 @@
 /* setBfree - DSP tonewheel organ
  *
  * Copyright (C) 2003-2004 Fredrik Kilander <fk@dsv.su.se>
- * Copyright (C) 2008-2012 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2008-2015 Robin Gareus <robin@gareus.org>
  * Copyright (C) 2012 Will Panther <pantherb@setbfree.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,9 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE
+#endif
 
 #include <stdio.h>
 #include <math.h>
@@ -28,6 +30,15 @@
 #include <sys/time.h>
 #include <unistd.h>
 
+#ifdef _WIN32
+#include <windows.h>
+#endif
+
+#ifdef WITH_SIGNATURE // gpg signed verified binary
+#include "gp3.h"
+#include WITH_SIGNATURE // pubkey
+#endif
+
 /* LV2 */
 #include "lv2/lv2plug.in/ns/lv2core/lv2.h"
 #include "lv2/lv2plug.in/ns/ext/atom/util.h"
@@ -46,8 +57,6 @@
 #include "pgmParser.h"
 #include "program.h"
 
-#define BUFFER_SIZE_SAMPLES  (128)
-
 typedef enum {
   B3S_MIDIIN = 0,
   B3S_MIDIOUT,
@@ -72,23 +81,36 @@ typedef struct {
   float bufA [BUFFER_SIZE_SAMPLES];
   float bufB [BUFFER_SIZE_SAMPLES];
   float bufC [BUFFER_SIZE_SAMPLES];
-  float bufJ [2][BUFFER_SIZE_SAMPLES];
+  float bufD [2][BUFFER_SIZE_SAMPLES]; // drum, tmp.
+  float bufL [2][BUFFER_SIZE_SAMPLES]; // leslie, out
 
   short suspend_ui_msg;
   short update_gui_now;
   short update_pgm_now;
   short swap_instances;
 
+  unsigned int active_keys [MAX_KEYS/32];
+
   struct b_instance *inst;
   struct b_instance *inst_offline;
+
+  char   lv2nfo[128];
+  uint32_t thirtysec;
+  uint32_t counter;
+  double   sin_phase;
+
 } B3S;
 
+
 enum {
   CMD_FREE    = 0,
   CMD_LOADPGM = 1,
   CMD_LOADCFG = 2,
   CMD_SAVEPGM = 3,
   CMD_SAVECFG = 4,
+  CMD_SETCFG  = 5,
+  CMD_PURGE   = 6,
+  CMD_RESET   = 7,
 };
 
 struct worknfo {
@@ -137,7 +159,6 @@ void initSynth(struct b_instance *inst, double rate) {
 
   initMidiTables(inst->midicfg);
 
-  setMIDINoteShift (inst->midicfg, 0);
   setDrawBars (inst, 0, defaultPreset);
 #if 0
   setDrawBars (inst, 1, defaultPreset);
@@ -169,7 +190,47 @@ freeSynth(struct b_instance *inst)
 #define MIN(A,B) (((A)<(B))?(A):(B))
 #endif
 
-uint32_t synthSound (B3S *instance, uint32_t written, uint32_t nframes, float **out) {
+#ifdef WITH_SIGNATURE
+static void scramble (B3S *b3s, float * const buf, const size_t n_samples)
+{
+	if (b3s->thirtysec == 0) return;
+	const uint32_t nc = b3s->counter + n_samples;
+	float p;
+
+	if (nc < b3s->thirtysec) {
+		b3s->counter += n_samples;
+		return;
+	}
+	if (nc > b3s->thirtysec + SampleRateD * 1) {
+		const float f = 8284.f / SampleRateD;
+		p = b3s->sin_phase;
+		for (int i=0; i < n_samples; ++i, p += f) {
+			const float g = i / (float)n_samples;
+			buf[i] = buf[i] * g + .125f * (1.f - g) * sinf(p);
+		}
+		b3s->counter = 0;
+		b3s->sin_phase = 0;
+		return;
+	} else if (b3s->counter >= b3s->thirtysec) {
+		const float f = 8284.f / SampleRateD;
+		p = b3s->sin_phase;
+		for (int i=0; i < n_samples; ++i, p += f) {
+			buf[i] = .125f * sinf(p);
+		}
+	} else {
+		const float f = 8284.f / SampleRateD;
+		p = b3s->sin_phase;
+		for (int i=0; i < n_samples; ++i, p += f) {
+			const float g = i / (float)n_samples;
+			buf[i] = buf[i] * (1.f-g) + .125f * g * sinf(p);
+		}
+	}
+	b3s->sin_phase = fmod(p, 2.0 * M_PI);
+	b3s->counter += n_samples;
+}
+#endif
+
+static uint32_t synthSound (B3S *instance, uint32_t written, uint32_t nframes, float **out) {
   B3S* b3s = (B3S*)instance;
 
   while (written < nframes) {
@@ -180,13 +241,17 @@ uint32_t synthSound (B3S *instance, uint32_t written, uint32_t nframes, float **
       oscGenerateFragment (instance->inst->synth, b3s->bufA, BUFFER_SIZE_SAMPLES);
       preamp (instance->inst->preamp, b3s->bufA, b3s->bufB, BUFFER_SIZE_SAMPLES);
       reverb (instance->inst->reverb, b3s->bufB, b3s->bufC, BUFFER_SIZE_SAMPLES);
-      whirlProc(instance->inst->whirl, b3s->bufC, b3s->bufJ[0], b3s->bufJ[1], BUFFER_SIZE_SAMPLES);
+#ifdef WITH_SIGNATURE
+      scramble (b3s, b3s->bufC, BUFFER_SIZE_SAMPLES);
+#endif
+      whirlProc3(instance->inst->whirl, b3s->bufC, b3s->bufL[0], b3s->bufL[1], b3s->bufD[0], b3s->bufD[1], BUFFER_SIZE_SAMPLES);
+
     }
 
     int nread = MIN(nremain, (BUFFER_SIZE_SAMPLES - b3s->boffset));
 
-    memcpy(&out[0][written], &b3s->bufJ[0][b3s->boffset], nread*sizeof(float));
-    memcpy(&out[1][written], &b3s->bufJ[1][b3s->boffset], nread*sizeof(float));
+    memcpy(&out[0][written], &b3s->bufL[0][b3s->boffset], nread*sizeof(float));
+    memcpy(&out[1][written], &b3s->bufL[1][b3s->boffset], nread*sizeof(float));
 
     written+=nread;
     b3s->boffset+=nread;
@@ -224,6 +289,8 @@ static void rc_cb(int fnid, const char *key, const char *kv, unsigned char val,
 #endif
   if (fnid >=0) {
     forge_kvcontrolmessage(&b3s->forge, &b3s->uris, key, (int32_t) val);
+  } else {
+    forge_kvconfigmessage(&b3s->forge, &b3s->uris, b3s->uris.sb3_cfgkv, key, kv);
   }
 }
 
@@ -236,7 +303,7 @@ static void pgm_cb(int num, int pc, const char *name, void *arg) {
 #endif
   LV2_Atom_Forge_Frame frame;
   lv2_atom_forge_frame_time(&b3s->forge, 0);
-  lv2_atom_forge_blank(&b3s->forge, &frame, 1, b3s->uris.sb3_midipgm);
+  x_forge_object(&b3s->forge, &frame, 1, b3s->uris.sb3_midipgm);
 
   lv2_atom_forge_property_head(&b3s->forge, b3s->uris.sb3_cckey, 0);
   lv2_atom_forge_int(&b3s->forge, pco);
@@ -257,7 +324,7 @@ static void mcc_cb(const char *fnname, const unsigned char chn, const unsigned c
 
   LV2_Atom_Forge_Frame frame;
   lv2_atom_forge_frame_time(&b3s->forge, 0);
-  lv2_atom_forge_blank(&b3s->forge, &frame, 1, b3s->uris.sb3_uimccset);
+  x_forge_object(&b3s->forge, &frame, 1, b3s->uris.sb3_uimccset);
 
   lv2_atom_forge_property_head(&b3s->forge, b3s->uris.sb3_cckey, 0);
   lv2_atom_forge_string(&b3s->forge, fnname, strlen(fnname));
@@ -278,8 +345,6 @@ void allocSynth(struct b_instance *inst) {
   initControllerTable (inst->midicfg);
 #if 1
   midiPrimeControllerMapping (inst->midicfg);
-#elif 0 // rg test midi-feedback
-  parseConfigurationFile (inst, "/home/rgareus/data/coding/setBfree/cfg/bcf2000.cfg");
 #endif
 
 }
@@ -294,7 +359,7 @@ static void rcstate_cb(int fnid, const char *key, const char *kv, unsigned char
   } else {
     sprintf(tmp, "M %s=%d\n", key, val);
   }
-  *cfg = realloc(*cfg, strlen(*cfg) + strlen(tmp) +1);
+  *cfg = (char*) realloc(*cfg, strlen(*cfg) + strlen(tmp) +1);
   strcat(*cfg, tmp);
 }
 
@@ -307,13 +372,25 @@ save(LV2_Handle                instance,
 {
   B3S* b3s = (B3S*)instance;
 
-  char *cfg = calloc(1, sizeof(char));
+  char *cfg = (char*) calloc(1, sizeof(char));
   rc_loop_state(b3s->inst->state, rcstate_cb, (void*) &cfg);
 
   int i;
   size_t rs = 0;
   char *out = NULL;
+
+#ifdef _WIN32
+  char temppath[MAX_PATH - 13];
+  char filename[MAX_PATH + 1];
+  if (0 == GetTempPath(sizeof(temppath), temppath))
+    return LV2_STATE_ERR_UNKNOWN;
+  if (0 == GetTempFileName(temppath, "sbfstate", 0, filename))
+    return LV2_STATE_ERR_UNKNOWN;
+  FILE *x = fopen(filename, "w+b");
+#else // POSIX
   FILE *x = open_memstream(&out, &rs);
+#endif
+
   for (i=0 ; i < 128; ++i) {
     int pgmNr = i + b3s->inst->progs->MIDIControllerPgmOffset;
     if (!(b3s->inst->progs->programmes[pgmNr].flags[0] & FL_INUSE)) {
@@ -324,7 +401,18 @@ save(LV2_Handle                instance,
   }
   fclose(x);
 
-  cfg = realloc(cfg, strlen(cfg) + strlen(out) +1);
+#ifdef _WIN32
+  x = fopen(filename, "rb");
+  fseek (x , 0 , SEEK_END);
+  long int rsize = ftell (x);
+  rewind(x);
+  out = (char*) malloc(rsize);
+  fread(out, sizeof(char), rsize, x);
+  fclose(x);
+  unlink(filename);
+#endif
+
+  cfg = (char*) realloc(cfg, strlen(cfg) + strlen(out) +1);
   strcat(cfg, out);
 
   store(handle, b3s->uris.sb3_state,
@@ -332,6 +420,7 @@ save(LV2_Handle                instance,
       b3s->uris.atom_String,
       LV2_STATE_IS_POD | LV2_STATE_IS_PORTABLE);
   free(cfg);
+  free(out);
   return LV2_STATE_SUCCESS;
 }
 
@@ -358,7 +447,7 @@ restore(LV2_Handle                  instance,
     return LV2_STATE_ERR_UNKNOWN;
   }
 
-  b3s->inst_offline = calloc(1, sizeof(struct b_instance));
+  b3s->inst_offline = (b_instance*) calloc(1, sizeof(struct b_instance));
   allocSynth(b3s->inst_offline);
 
   const char* cfg = (const char*)value;
@@ -421,6 +510,24 @@ static void rcsave_cb(int fnid, const char *key, const char *kv, unsigned char v
   }
 }
 
+static void clone_cb_cfg(int fnid, const char *key, const char *kv, unsigned char val, void *arg) {
+  if (fnid < 0) {
+    evaluateConfigKeyValue((struct b_instance *) arg, key, kv);
+  }
+}
+
+static void clone_map_cb(int fnid, const char *key, const char *kv, unsigned char val, void *arg) {
+  if (fnid < 0 && !strncmp(key, "midi.controller.", 16)) {
+    evaluateConfigKeyValue((struct b_instance *) arg, key, kv);
+  }
+}
+
+static void clone_cb_mcc(int fnid, const char *key, const char *kv, unsigned char val, void *arg) {
+  if (fnid >= 0) {
+    callMIDIControlFunction(((struct b_instance *) arg)->midicfg, key, val);
+  }
+}
+
 /* LV2 -- worker */
 static LV2_Worker_Status
 work(LV2_Handle                  instance,
@@ -438,6 +545,59 @@ work(LV2_Handle                  instance,
   struct worknfo *w = (struct worknfo*) data;
 
   switch(w->cmd) {
+    case CMD_PURGE:
+      if (b3s->inst_offline) {
+	// this should not happen
+	fprintf(stderr, "B3LV2: re-init in progress\n");
+	w->status = -1;
+      } else {
+	fprintf(stderr, "B3LV2: reinitialize\n");
+	b3s->inst_offline = (b_instance*) calloc(1, sizeof(struct b_instance));
+	allocSynth(b3s->inst_offline);
+	// clone midi map only
+	rc_loop_state(b3s->inst->state, clone_map_cb, b3s->inst_offline);
+	// copy program info
+	memcpy(b3s->inst_offline->progs,  b3s->inst->progs, sizeof(struct b_programme));
+	initSynth(b3s->inst_offline, SampleRateD);
+	// replay CCs after synth init
+	rc_loop_state(b3s->inst->state, clone_cb_mcc, b3s->inst_offline);
+	w->status = 0;
+      }
+      break;
+    case CMD_RESET:
+      if (b3s->inst_offline) {
+	// this should not happen
+	fprintf(stderr, "B3LV2: reset ignored. re-init in progress\n");
+	w->status = -1;
+      } else {
+	fprintf(stderr, "B3LV2: factory reset\n");
+	b3s->inst_offline = (b_instance*) calloc(1, sizeof(struct b_instance));
+	allocSynth(b3s->inst_offline);
+	initSynth(b3s->inst_offline, SampleRateD);
+	w->status = 0;
+      }
+      break;
+    case CMD_SETCFG:
+      if (b3s->inst_offline) {
+	// this should not happen
+	fprintf(stderr, "B3LV2: setcfg ignored. re-init in progress\n");
+	w->status = -1;
+      } else {
+	// fprintf(stderr, "B3LV2: adding cfg line: %s\n", w->msg);
+	b3s->inst_offline = (b_instance*) calloc(1, sizeof(struct b_instance));
+	allocSynth(b3s->inst_offline);
+	// clone current state...
+	rc_loop_state(b3s->inst->state, clone_cb_cfg, b3s->inst_offline);
+	// copy program info
+	memcpy(b3s->inst_offline->progs,  b3s->inst->progs, sizeof(struct b_programme));
+	// add user-config
+	parseConfigurationLine (b3s->inst_offline, "LV2", 0, w->msg);
+	initSynth(b3s->inst_offline, SampleRateD);
+	// replay CCs after synth init
+	rc_loop_state(b3s->inst->state, clone_cb_mcc, b3s->inst_offline);
+	w->status = 0;
+      }
+      break;
     case CMD_LOADPGM:
       fprintf(stderr, "B3LV2: loading pgm file: %s\n", w->msg);
       if (!(w->status=loadProgrammeFile(b3s->inst->progs, w->msg))) {
@@ -447,10 +607,10 @@ work(LV2_Handle                  instance,
     case CMD_LOADCFG:
       if (b3s->inst_offline) {
 	fprintf(stderr, "B3LV2: restore ignored. re-init in progress\n");
-	return LV2_STATE_ERR_UNKNOWN;
+	return LV2_WORKER_ERR_UNKNOWN;
       }
       fprintf(stderr, "B3LV2: loading cfg file: %s\n", w->msg);
-      b3s->inst_offline = calloc(1, sizeof(struct b_instance));
+      b3s->inst_offline = (b_instance*) calloc(1, sizeof(struct b_instance));
       allocSynth(b3s->inst_offline);
       w->status = parseConfigurationFile (b3s->inst_offline, w->msg);
       initSynth(b3s->inst_offline, SampleRateD);
@@ -500,7 +660,7 @@ work(LV2_Handle                  instance,
 static void forge_message_str(B3S *b3s, LV2_URID uri, const char *msg) {
   LV2_Atom_Forge_Frame frame;
   lv2_atom_forge_frame_time(&b3s->forge, 0);
-  lv2_atom_forge_blank(&b3s->forge, &frame, 1, uri);
+  x_forge_object(&b3s->forge, &frame, 1, uri);
   lv2_atom_forge_property_head(&b3s->forge, b3s->uris.sb3_uimsg, 0);
   lv2_atom_forge_string(&b3s->forge, msg, strlen(msg));
   lv2_atom_forge_pop(&b3s->forge, &frame);
@@ -521,6 +681,27 @@ work_response(LV2_Handle  instance,
   struct worknfo *w = (struct worknfo*) data;
 
   switch(w->cmd) {
+    case CMD_PURGE:
+    case CMD_RESET:
+      if (w->status){
+	// this should not happen
+	sprintf(tmp, "error modyfing CFG. Organ is busy.");
+      } else {
+	sprintf(tmp, "%s executed successfully.",
+	    (w->cmd == CMD_RESET) ? "Factory-reset" : "Reconfigure");
+	b3s->swap_instances = 1;
+      }
+      forge_message_str(b3s, b3s->uris.sb3_uimsg, tmp);
+      break;
+    case CMD_SETCFG:
+      if (w->status){
+	// this should not happen
+	sprintf(tmp, "error modyfing CFG. Organ is busy.");
+	forge_message_str(b3s, b3s->uris.sb3_uimsg, tmp);
+      } else {
+	b3s->swap_instances = 1;
+      }
+      break;
     case CMD_LOADCFG:
       b3s->swap_instances = 1;
       if (w->status)
@@ -575,6 +756,7 @@ postrun (B3S* b3s)
 
     /* hide midi-maps, stop possibly pending midi-bind process */
     forge_kvcontrolmessage(&b3s->forge, &b3s->uris, "special.midimap", (int32_t) 0);
+    forge_kvcontrolmessage(&b3s->forge, &b3s->uris, "special.reinit", (int32_t) 1);
 
     b3s->schedule->schedule_work(b3s->schedule->handle, sizeof(struct worknfo), &w);
     b3s->update_gui_now = 1;
@@ -588,11 +770,33 @@ static void iowork(B3S* b3s, const LV2_Atom_Object* obj, int cmd) {
   if (name) {
     struct worknfo w;
     w.cmd = cmd;
+    w.status = -1;;
     strncpy(w.msg, (char *)LV2_ATOM_BODY(name), 1024);
     b3s->schedule->schedule_work(b3s->schedule->handle, sizeof(struct worknfo), &w);
   }
 }
 
+static void advanced_config_set(B3S* b3s, const LV2_Atom_Object* obj) {
+  const LV2_Atom* cfgline = NULL;
+  lv2_atom_object_get(obj, b3s->uris.sb3_cckey, &cfgline, 0);
+  if (cfgline) {
+    struct worknfo w;
+    char* msg = (char *)LV2_ATOM_BODY(cfgline);
+    if(!strcmp("special.reset=1", msg)) {
+      w.cmd = CMD_RESET;
+    }
+    else if(!strcmp("special.reconfigure=1", msg)) {
+      w.cmd = CMD_PURGE;
+    }
+    else {
+      w.cmd = CMD_SETCFG;
+    }
+    w.status = -1;
+    strncpy(w.msg, msg, 1024);
+    b3s->schedule->schedule_work(b3s->schedule->handle, sizeof(struct worknfo), &w);
+  }
+}
+
 /* main LV2 */
 
 static LV2_Handle
@@ -635,13 +839,128 @@ instantiate(const LV2_Descriptor*     descriptor,
   b3s->update_gui_now = 0;
   b3s->update_pgm_now = 0;
 
-  b3s->inst = calloc(1, sizeof(struct b_instance));
+  b3s->inst = (b_instance*) calloc(1, sizeof(struct b_instance));
   b3s->inst_offline = NULL;
 
   allocSynth(b3s->inst);
+
+#ifdef JACK_DESCRIPT
+  // CODE DUP src/main.c
+  char * defaultConfigFile = NULL;
+  char * defaultProgrammeFile = NULL;
+
+#ifdef _WIN32
+  // out of luck with CSIDL_LOCAL_APPDATA
+  const char * homedrive = getenv("HOMEDRIVE");
+  const char * homepath = getenv("HOMEPATH");
+  if (homedrive && homepath && (strlen(homedrive) + strlen(homepath) + 38) < 1024) {
+    size_t hl = strlen(homedrive) + strlen(homepath);
+    defaultConfigFile=(char*) malloc(hl+38);
+    defaultProgrammeFile=(char*) malloc(hl+38);
+    sprintf(defaultConfigFile,    "%s%s\\Local Settings\\setBfree\\default.cfg", homedrive, homepath);
+    sprintf(defaultProgrammeFile, "%s%s\\Local Settings\\setBfree\\default.pgm", homedrive, homepath);
+  }
+#else // unices: prefer XDG_CONFIG_HOME
+  if (getenv("XDG_CONFIG_HOME")) {
+    size_t hl = strlen(getenv("XDG_CONFIG_HOME"));
+    defaultConfigFile=(char*) malloc(hl+22);
+    defaultProgrammeFile=(char*) malloc(hl+22);
+    sprintf(defaultConfigFile,    "%s/setBfree/default.cfg", getenv("XDG_CONFIG_HOME"));
+    sprintf(defaultProgrammeFile, "%s/setBfree/default.pgm", getenv("XDG_CONFIG_HOME"));
+  }
+  else if (getenv("HOME")) {
+    size_t hl = strlen(getenv("HOME"));
+# ifdef __APPLE__
+    defaultConfigFile=(char*) malloc(hl+42);
+    defaultProgrammeFile=(char*) malloc(hl+42);
+    sprintf(defaultConfigFile,    "%s/Library/Preferences/setBfree/default.cfg", getenv("HOME"));
+    sprintf(defaultProgrammeFile, "%s/Library/Preferences/setBfree/default.pgm", getenv("HOME"));
+# else // linux, BSD, etc
+    defaultConfigFile=(char*) malloc(hl+30);
+    defaultProgrammeFile=(char*) malloc(hl+30);
+    sprintf(defaultConfigFile,    "%s/.config/setBfree/default.cfg", getenv("HOME"));
+    sprintf(defaultProgrammeFile, "%s/.config/setBfree/default.pgm", getenv("HOME"));
+# endif
+  }
+#endif
+
+  if (access (defaultConfigFile, R_OK) == 0) {
+    parseConfigurationFile (b3s->inst, defaultConfigFile);
+  }
+#endif
+
   setControlFunctionCallback(b3s->inst->midicfg, mctl_cb, b3s);
   initSynth(b3s->inst, rate);
 
+#ifdef JACK_DESCRIPT
+  if (access (defaultProgrammeFile, R_OK) == 0) {
+    loadProgrammeFile (b3s->inst->progs, defaultProgrammeFile);
+  }
+  free(defaultConfigFile);
+  free(defaultProgrammeFile);
+#endif
+
+  strcpy(b3s->lv2nfo, "v" VERSION);
+  b3s->thirtysec = 0;
+#ifdef WITH_SIGNATURE
+  {
+    b3s->thirtysec = 30 * rate;
+    b3s->counter = 0;
+    b3s->sin_phase = 0;
+
+    gp3_initialize ();
+    load_master_key (); // in header WITH_SIGNATURE
+    gp3_loglevel (GP3L_SILENT);
+    int rc = -1;
+    char signature_file0[1024] = "";
+    char signature_file1[1024] = "";
+#ifdef _WIN32
+	ExpandEnvironmentStrings("%localappdata%\\"SIGFILE, signature_file0, 1024);
+	ExpandEnvironmentStrings("%localappdata%\\x42_license.txt", signature_file1, 1024);
+#else
+	const char * home = getenv("HOME");
+	if (home && (strlen(home) + strlen(SIGFILE) + 3) < 1024) {
+		sprintf(signature_file0, "%s/.%s", home, SIGFILE);
+	}
+	if (home && (strlen(home) + 18) < 1024) {
+		sprintf(signature_file1, "%s/.x42_license.txt", home);
+	}
+#endif
+	if (!access(signature_file0, R_OK)) {
+	  rc = gp3_checksigfile (signature_file0);
+	} else if (!access(signature_file1, R_OK)) {
+	  rc = gp3_checksigfile (signature_file1);
+	}
+	if (rc == 0) {
+	  bool ok = false;
+	  char data[8192];
+	  char *tmp=NULL;
+	  uint32_t len = gp3_get_text(data, sizeof(data));
+	  if (len == sizeof(data)) data[sizeof(data)-1] = '\0';
+	  else data[len] = '\0';
+	  if ((tmp = strchr(data, '\n'))) *tmp = 0;
+	  b3s->lv2nfo[sizeof(b3s->lv2nfo) - 1] = 0;
+	  if (tmp++ && *tmp) {
+	    if ((tmp = strstr(tmp, SB3_URI))) {
+	      char *t1, *t2;
+	      ok = true;
+	      t1 = tmp + 1 + strlen(SB3_URI);
+	      t2 = strchr(t1, '\n');
+	      if (t2) { *t2 = 0; }
+	      if (strlen(t1) > 0 && strncmp(t1, VERSION, strlen(t1))) {
+	      ok = false;
+	      }
+	    }
+	  }
+	  if (ok) {
+	    b3s->thirtysec = 0;
+	    strncat(b3s->lv2nfo, " ", sizeof(b3s->lv2nfo) - strlen(b3s->lv2nfo));
+	    strncat(b3s->lv2nfo, data, sizeof(b3s->lv2nfo) - strlen(b3s->lv2nfo));
+	  }
+	}
+	gp3_cleanup ();
+  }
+#endif
   return (LV2_Handle)b3s;
 }
 
@@ -701,11 +1020,11 @@ run(LV2_Handle instance, uint32_t n_samples)
 	if (written + BUFFER_SIZE_SAMPLES < ev->time.frames
 	    && ev->time.frames < n_samples) {
 	  /* first syntheize sound up until the message timestamp */
-	  written = synthSound(instance, written, ev->time.frames, audio);
+	  written = synthSound(b3s, written, ev->time.frames, audio);
 	}
 	/* send midi message to synth, CC's will trigger hook -> update GUI */
 	parse_raw_midi_data(b3s->inst, (uint8_t*)(ev+1), ev->body.size);
-      } else if (ev->body.type == b3s->uris.atom_Blank) {
+      } else if (ev->body.type == b3s->uris.atom_Blank || ev->body.type == b3s->uris.atom_Object) {
 	/* process messages from GUI */
 	const LV2_Atom_Object* obj = (LV2_Atom_Object*)&ev->body;
 	if (obj->body.otype == b3s->uris.sb3_uiinit) {
@@ -713,10 +1032,11 @@ run(LV2_Handle instance, uint32_t n_samples)
 	} else if (obj->body.otype == b3s->uris.sb3_uimccquery) {
 	  midi_loopCCAssignment(b3s->inst->midicfg, 7, mcc_cb, b3s);
 	} else if (obj->body.otype == b3s->uris.sb3_uimccset) {
-	  const LV2_Atom* key = NULL;
-	  lv2_atom_object_get(obj, b3s->uris.sb3_cckey, &key, 0);
-	  if (key) {
-	    midi_uiassign_cc(b3s->inst->midicfg, (const char*)LV2_ATOM_BODY(key));
+	  const LV2_Atom* cmd = NULL;
+	  const LV2_Atom* flags = NULL;
+	  lv2_atom_object_get(obj, b3s->uris.sb3_cckey, &flags, b3s->uris.sb3_ccval, &cmd, 0);
+	  if (cmd && flags) {
+	    midi_uiassign_cc(b3s->inst->midicfg, (const char*)LV2_ATOM_BODY(cmd), ((LV2_Atom_Int*)flags)->body);
 	  }
 	} else if (obj->body.otype == b3s->uris.sb3_midipgm) {
 	  const LV2_Atom* key = NULL;
@@ -740,6 +1060,10 @@ run(LV2_Handle instance, uint32_t n_samples)
 	  iowork(b3s, obj, CMD_SAVEPGM);
 	} else if (obj->body.otype == b3s->uris.sb3_savecfg) {
 	  iowork(b3s, obj, CMD_SAVECFG);
+	} else if (obj->body.otype == b3s->uris.sb3_cfgstr) {
+	  if (!b3s->inst_offline) {
+	    advanced_config_set(b3s, obj);
+	  }
 	} else if (obj->body.otype == b3s->uris.sb3_control) {
 	  b3s->suspend_ui_msg = 1;
 	  const LV2_Atom_Object* obj = (LV2_Atom_Object*)&ev->body;
@@ -757,22 +1081,41 @@ run(LV2_Handle instance, uint32_t n_samples)
     }
   }
 
+  /* synthesize [remaining] sound */
+  synthSound(b3s, written, n_samples, audio);
+
+  /* send active keys to GUI - IFF changed */
+  bool keychanged = false;
+  for (int i = 0 ; i < MAX_KEYS/32; ++i) {
+    if (b3s->active_keys[i] != b3s->inst->synth->_activeKeys[i]) {
+      keychanged = true;
+    }
+    b3s->active_keys[i] = b3s->inst->synth->_activeKeys[i];
+  }
+  if (keychanged) {
+    LV2_Atom_Forge_Frame frame;
+    lv2_atom_forge_frame_time(&b3s->forge, 0);
+    x_forge_object(&b3s->forge, &frame, 1, b3s->uris.sb3_activekeys);
+    lv2_atom_forge_property_head(&b3s->forge, b3s->uris.sb3_keyarrary, 0);
+    lv2_atom_forge_vector(&b3s->forge, sizeof(unsigned int), b3s->uris.atom_Int, MAX_KEYS/32, b3s->active_keys);
+    lv2_atom_forge_pop(&b3s->forge, &frame);
+  }
+
+  /* check for new instances */
+  postrun(b3s);
+
   if (b3s->update_gui_now) {
     b3s->update_gui_now = 0;
     b3s->update_pgm_now = 1;
     b3s->suspend_ui_msg = 1;
     rc_loop_state(b3s->inst->state, rc_cb, b3s);
     b3s->suspend_ui_msg = 0;
+    forge_kvconfigmessage(&b3s->forge, &b3s->uris, b3s->uris.sb3_cfgkv, "lv2.info", b3s->lv2nfo);
+    forge_kvcontrolmessage(&b3s->forge, &b3s->uris, "special.init", (int32_t) b3s->thirtysec);
   } else if (b3s->update_pgm_now) {
     b3s->update_pgm_now = 0;
     loopProgammes(b3s->inst->progs, 1, pgm_cb, b3s);
   }
-
-  /* synthesize [remaining] sound */
-  synthSound(instance, written, n_samples, audio);
-
-  /* check for new instances */
-  postrun(b3s);
 }
 
 static void
