@@ -1,7 +1,7 @@
 /* setBfree - DSP tonewheel organ
  *
  * Copyright (C) 2003-2004 Fredrik Kilander <fk@dsv.su.se>
- * Copyright (C) 2008-2012 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2008-2015 Robin Gareus <robin@gareus.org>
  * Copyright (C) 2012 Will Panther <pantherb@setbfree.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,7 @@
 
 #include "main.h"
 #include "global_inst.h"
+#include "pgmParser.h"
 
 #ifdef HAVE_ZITACONVOLVE
 #include "convolution.h"
@@ -96,7 +97,7 @@ static int distributeParameter (b_instance* inst, ConfigContext * cfg) {
 /*
  *
  */
-void parseConfigurationLine (void *inst, char * fname, int lineNumber, char * oneLine) {
+void parseConfigurationLine (void *inst, const char * fname, int lineNumber, char * oneLine) {
   char delim[] = "=\n";
   char * s = oneLine;
   char * name;
@@ -116,40 +117,39 @@ void parseConfigurationLine (void *inst, char * fname, int lineNumber, char * on
   if ((name = strtok (s, delim)) != NULL) {
     int i;
 
-	for (i = strlen (name) - 1; isspace (name[i]); name[i] = '\0', i--);
+    for (i = strlen (name) - 1; isspace (name[i]); name[i] = '\0', i--);
 
-	if ((value = strtok (NULL, delim)) != NULL) {
-	  char * t;
-	  while (isspace (value[0])) value++;
-	  for (t = value; *t != '\0'; t++) {
-	    if (*t == '#') {	/* Terminate value at comment */
-	      *t = '\0';
-	      break;
-	    }
-	  }
-	}
-	else {
-	  value = "";
-	}
-
-	i = strlen (value);
-	if (0 < i) {
-	  for (i = i - 1;  isspace (value[i]); value[i] = '\0', i--);
-	}
-
-	if (strcasecmp (name, "config.read") == 0) {
-	  parseConfigurationFile (inst, value);
-	}
-	else {
-	  ConfigContext cfg;
-	  cfg.fname  = fname;
-	  cfg.linenr = lineNumber;
-	  cfg.name   = name;
-	  cfg.value  = value;
-	  distributeParameter ((b_instance*) inst, & cfg);
+    if ((value = strtok (NULL, delim)) != NULL) {
+      char * t;
+      while (isspace (value[0])) value++;
+      for (t = value; *t != '\0'; t++) {
+	if (*t == '#') {	/* Terminate value at comment */
+	  *t = '\0';
+	  break;
 	}
       }
 
+      i = strlen (value);
+      if (0 < i) {
+	for (i = i - 1; isspace (value[i]); value[i] = '\0', i--);
+      }
+    }
+
+    if (strcasecmp (name, "config.read") == 0) {
+      parseConfigurationFile (inst, value);
+    }
+    else if (strcasecmp (name, "program.read") == 0) {
+      loadProgrammeFile (((b_instance*)inst)->progs, value);
+    }
+    else {
+      ConfigContext cfg;
+      cfg.fname  = fname;
+      cfg.linenr = lineNumber;
+      cfg.name   = name;
+      cfg.value  = value ? value : "";
+      distributeParameter ((b_instance*) inst, & cfg);
+    }
+  }
 }
 
 int evaluateConfigKeyValue(void *inst, const char *key, const char *value) {
@@ -163,12 +163,12 @@ int evaluateConfigKeyValue(void *inst, const char *key, const char *value) {
 
 #ifndef CFG_MAIN
 /* text representation of enum conftype */
-const char *conftypenames[CFG_LAST] = { "S", "D", "F", "I" };
+const char *conftypenames[CFG_LAST] = { "S", "D", "D", "F", "I" };
 
 /*
  *
  */
-void formatDoc (char *modulename, const ConfigDoc *d) {
+static void formatDoc (const char *modulename, const ConfigDoc *d) {
   printf("Parameters for '%s':\n", modulename);
   while (d && d->name) {
     if (strlen(d->name) >= 40) {
@@ -255,6 +255,10 @@ void dumpConfigDoc () {
   "   <22:vibrato1, <44:chorus1, <66:vibrato2, <88:chorus2, <110vibrato3, >=110:chorus3\n"
   "  vibrato.routing                            I* (0)\n"
   "    <32:off, <64:lower, <96:upper, >=96:both\n"
+  "  vibrato.upper                              I* (0)\n"
+  "    <64:off, >=64 on                        \n"
+  "  vibrato.lower                              I* (0)\n"
+  "    <64:off, >=64 on                        \n"
   "  percussion.enable                          I* (0)\n"
   "    <16:off, <63:normal, <112:soft, >=112:off\n"
   "  percussion.decay                           I* (0)\n"
@@ -273,7 +277,7 @@ void dumpConfigDoc () {
 /*
  *
  */
-int parseConfigurationFile (void *inst, char * fname) {
+int parseConfigurationFile (void *inst, const char * fname) {
   int lineNumber = 0;
   char lineBuf [LINEBUFSZ];
   FILE * fp;
@@ -298,7 +302,7 @@ int parseConfigurationFile (void *inst, char * fname) {
 /*
  *
  */
-void showConfigfileContext (ConfigContext * cfg, char * msg) {
+void showConfigfileContext (ConfigContext * cfg, const char * msg) {
 
   assert (cfg != NULL);
   assert (msg != NULL);
@@ -432,7 +436,7 @@ const char * getConfigValue (ConfigContext * cfg) {
  *             the context, 1 if it did and an assignment was made
  *             -1 if the name matched but no assignment was made.
  */
-int getConfigParameter_d (char * par, ConfigContext * cfg, double * dp) {
+int getConfigParameter_d (const char * par, ConfigContext * cfg, double * dp) {
 
   assert (par != NULL);
   assert (cfg != NULL);
@@ -464,7 +468,7 @@ int getConfigParameter_d (char * par, ConfigContext * cfg, double * dp) {
  *             the context, 1 if it did and an assignment was made
  *             -1 if the name matched but no assignment was made.
  */
-int getConfigParameter_i (char * par, ConfigContext * cfg, int * ip) {
+int getConfigParameter_i (const char * par, ConfigContext * cfg, int * ip) {
 
   assert (par != NULL);
   assert (cfg != NULL);
@@ -496,7 +500,7 @@ int getConfigParameter_i (char * par, ConfigContext * cfg, int * ip) {
  *             the context, 1 if it did and an assignment was made
  *             -1 if the name matched but no assignment was made.
  */
-int getConfigParameter_f (char * par, ConfigContext * cfg, float * fp) {
+int getConfigParameter_f (const char * par, ConfigContext * cfg, float * fp) {
 
   assert (par != NULL);
   assert (cfg != NULL);
@@ -527,7 +531,7 @@ int getConfigParameter_f (char * par, ConfigContext * cfg, float * fp) {
  *             the context, 1 if it did and an assignment was made
  *             -1 if the name matched but no assignment was made.
  */
-int getConfigParameter_ir (char * par,
+int getConfigParameter_ir (const char * par,
 			   ConfigContext * cfg,
 			   int * ip,
 			   int lowInc,
@@ -560,7 +564,7 @@ int getConfigParameter_ir (char * par,
  *             the context, 1 if it did and an assignment was made
  *             -1 if the name matched but no assignment was made.
  */
-int getConfigParameter_dr (char * par,
+int getConfigParameter_dr (const char * par,
 			   ConfigContext * cfg,
 			   double * dp,
 			   double lowInc,
@@ -590,7 +594,7 @@ int getConfigParameter_dr (char * par,
  *             the context, 1 if it did and an assignment was made
  *             -1 if the name matched but no assignment was made.
  */
-int getConfigParameter_fr (char * par,
+int getConfigParameter_fr (const char * par,
 			   ConfigContext * cfg,
 			   float * fp,
 			   float lowInc,
