@@ -1,7 +1,7 @@
 /* setBfree - DSP tonewheel organ
  *
  * Copyright (C) 2003-2004 Fredrik Kilander <fk@dsv.su.se>
- * Copyright (C) 2008-2012 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2008-2015 Robin Gareus <robin@gareus.org>
  * Copyright (C) 2012 Will Panther <pantherb@setbfree.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,11 +17,17 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
+#if defined _WIN32 && !defined WIN32
+#define WIN32 // jack2 sysdeps
+#endif
+
+#define _XOPEN_SOURCE 700
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/ioctl.h>
+#include <strings.h>
+#include <stdint.h>
 #include <fcntl.h>
 #include <ctype.h>
 #include <math.h>
@@ -30,12 +36,20 @@
 #include <pthread.h>
 #include <limits.h>
 #include <getopt.h>
+
+#ifdef USE_WEAK_JACK
+#include "weakjack/weak_libjack.h"
+#else
 #include <jack/jack.h>
 #include <jack/midiport.h>
-#include <sys/mman.h>
+#endif
 
 #ifndef _WIN32
+#include <sys/ioctl.h>
+#include <sys/mman.h>
 #include <signal.h>
+#else
+#include <windows.h>
 #endif
 #include "global_inst.h"
 #include "vibrato.h"
@@ -151,7 +165,7 @@ int jack_audio_callback (jack_nframes_t nframes, void *arg) {
 
   if (!synth_ready) {
     for (i=0;i<AUDIO_CHANNELS;i++) {
-      out[i] = jack_port_get_buffer (j_output_port[i], nframes);
+      out[i] = (jack_default_audio_sample_t*) jack_port_get_buffer (j_output_port[i], nframes);
       memset(out[i], 0, nframes*sizeof(jack_default_audio_sample_t));
     }
     return (0);
@@ -163,7 +177,7 @@ int jack_audio_callback (jack_nframes_t nframes, void *arg) {
   }
 
   for (i=0;i<AUDIO_CHANNELS;i++) {
-    out[i] = jack_port_get_buffer (j_output_port[i], nframes);
+    out[i] = (jack_default_audio_sample_t*) jack_port_get_buffer (j_output_port[i], nframes);
   }
 
   static int boffset = BUFFER_SIZE_SAMPLES;
@@ -244,8 +258,8 @@ int open_jack(void) {
   jack_set_process_callback(j_client,jack_audio_callback, &inst);
   jack_set_sample_rate_callback (j_client, jack_srate_callback, NULL);
 
-  j_output_port= calloc(AUDIO_CHANNELS,sizeof(jack_port_t*));
-  j_output_bufferptrs = calloc(AUDIO_CHANNELS,sizeof(jack_default_audio_sample_t*));
+  j_output_port = (jack_port_t**) calloc(AUDIO_CHANNELS,sizeof(jack_port_t*));
+  j_output_bufferptrs = (jack_default_audio_sample_t**) calloc(AUDIO_CHANNELS,sizeof(jack_default_audio_sample_t*));
 
   for (i=0;i<AUDIO_CHANNELS;i++) {
 #if 0
@@ -446,6 +460,8 @@ static void Usage (int configdoc) {
   "  -D, --noCC        do not load default CC map on startup\n"
   "  -h                Print short help text\n"
   "  -H, --help        Print complete help text with parameter list\n"
+  "  -l <pgm>          Load a MIDI program-preset at startup\n"
+  "                    (may override previous settings given with -r, -U,..)\n"
   "  -M <filename>, --midnam <filename>\n"
   "                    export current controller mapping to .midnam file\n"
   "  -p <filename>, --program <filename>\n"
@@ -453,6 +469,9 @@ static void Usage (int configdoc) {
   "  -P, --noprogram   Do not read the default program file\n"
   "                    the built-in programs are cleared as well\n"
   "  -r, --randomize   Randomize initial preset (whacky but true)\n"
+  "  -U, --upper <drawbar settings>\n"
+  "                    Specify initial drawbar settings, for the upper\n"
+  "                    manual as 9 digits. e.g. 808000000\n"
   "  -V, --version     Print version information\n"
   "\n");
   if (configdoc) {
@@ -512,6 +531,8 @@ static void Usage (int configdoc) {
   printf (
   "%s jack.out.left=system:playback_7 jack.out.right=system:playback_8\n", name);
   printf (
+  "%s -U 868000000\n", name);
+  printf (
   "\n"
   "Report bugs at <http://github.com/pantherb/setBfree/issues>.\n"
   "Website and manual: <http://setbfree.org>\n"
@@ -522,7 +543,7 @@ static void PrintVersion () {
   printf ("%s %s\n\n", name, VERSION);
   printf(
 "Copyright (C) 2003-2004 Fredrik Kilander <fk@dsv.su.se>\n"
-"Copyright (C) 2008-2012 Robin Gareus <robin@gareus.org>\n"
+"Copyright (C) 2008-2014 Robin Gareus <robin@gareus.org>\n"
 "Copyright (C) 2010 Ken Restivo <ken@restivo.org>\n"
 "Copyright (C) 2012 Will Panther <pantherb@setbfree.org>\n"
 "\n"
@@ -532,12 +553,12 @@ static void PrintVersion () {
 }
 
 static const ConfigDoc doc[] = {
-  {"midi.driver", CFG_TEXT, "\"jack\"", "The midi driver to use, 'jack' or 'alsa'"},
-  {"midi.port", CFG_TEXT, "\"\"", "The midi port(s) to auto-connect to. With alsa it's a single port-name or number, jack accepts regular expressions."},
-  {"jack.connect", CFG_TEXT, "\"system:playback_\"", "Auto connect both audio-ports to a given regular-expression. This setting is ignored if either of jack.out.{left|right} is specified."},
-  {"jack.out.left", CFG_TEXT, "\"\"", "Connect left-output to this jack-port (exact name)"},
-  {"jack.out.right", CFG_TEXT, "\"\"", "Connect right-output to this jack-port (exact name)"},
-  {NULL}
+  {"midi.driver", CFG_TEXT, "\"jack\"", "The midi driver to use, 'jack' or 'alsa'", INCOMPLETE_DOC},
+  {"midi.port", CFG_TEXT, "\"\"", "The midi port(s) to auto-connect to. With alsa it's a single port-name or number, jack accepts regular expressions.", INCOMPLETE_DOC},
+  {"jack.connect", CFG_TEXT, "\"system:playback_\"", "Auto connect both audio-ports to a given regular-expression. This setting is ignored if either of jack.out.{left|right} is specified.", INCOMPLETE_DOC},
+  {"jack.out.left", CFG_TEXT, "\"\"", "Connect left-output to this jack-port (exact name)", INCOMPLETE_DOC},
+  {"jack.out.right", CFG_TEXT, "\"\"", "Connect right-output to this jack-port (exact name)", INCOMPLETE_DOC},
+  DOC_SENTINEL
 };
 
 int mainConfig (ConfigContext * cfg) {
@@ -588,6 +609,19 @@ const ConfigDoc *mainDoc () {
   return doc;
 }
 
+static void parse_preset(unsigned int *p, const char *c) {
+  unsigned int i;
+  for (i=0; i < strlen(c); ++i) {
+    if (c[i] < '0' || c[i] > '9') {
+      p[i] = 0;
+    } else {
+      p[i] = c[i] - '0';
+    }
+  }
+  for (;i < 9; ++i) {
+    p[i] = 0;
+  }
+}
 /*
  * Main program.
  */
@@ -598,7 +632,8 @@ int main (int argc, char * argv []) {
   int printCCTable = FALSE;
   int doDefaultCC = TRUE;
   char * configOverride [NOF_CFG_OVERS];
-  size_t configOverEnd = 0;
+  int configOverEnd = 0;
+  int loadProgram = -1;
   unsigned int randomPreset[9];
   unsigned int defaultPreset[9] = {8,8,8, 0,0,0,0, 0,0};
   unsigned int * presetSelect = defaultPreset;
@@ -615,8 +650,8 @@ int main (int argc, char * argv []) {
     jack_port[i] = NULL;
   jack_ports = strdup("system:playback_");
 
-  const char *optstring = "c:CdDhHM:p:PrV";
-  struct option long_options[] = {
+  const char *optstring = "c:CdDhHl:M:p:PrU:V";
+  const struct option long_options[] = {
     { "help",       no_argument,       0, 'H' },
     { "program",    required_argument, 0, 'p' },
     { "config",     required_argument, 0, 'c' },
@@ -626,6 +661,7 @@ int main (int argc, char * argv []) {
     { "midnam",     required_argument, 0, 'M' },
     { "noprogram",  no_argument,       0, 'P' },
     { "randomize",  no_argument,       0, 'r' },
+    { "upper",      required_argument, 0, 'U' },
     { "version",    no_argument,       0, 'V' },
     { 0, 0, 0, 0 }
   };
@@ -652,6 +688,9 @@ int main (int argc, char * argv []) {
 	Usage(1);
 	return (0);
 	break;
+      case 'l':
+	loadProgram = atoi(optarg);
+	break;
       case 'M':
 	midnam = optarg;
 	break;
@@ -678,6 +717,9 @@ int main (int argc, char * argv []) {
       case 'P':
 	doDefaultProgram = FALSE;
 	break;
+      case 'U':
+	parse_preset(defaultPreset, optarg);
+	break;
       case 'V':
 	PrintVersion();
 	return(0);
@@ -688,7 +730,7 @@ int main (int argc, char * argv []) {
     }
   }
 
-  for (i = optind; i < argc; i++) {
+  for (i = optind; i < argc; ++i) {
     char * av = argv[i];
     if (strchr (av, '=') != NULL) {
       /* Remember this as a config parameter */
@@ -771,10 +813,11 @@ int main (int argc, char * argv []) {
   /*
    * Having configured the initialization phase we can now actually do it.
    */
-
+#ifndef _WIN32
   if (mlockall (MCL_CURRENT | MCL_FUTURE)) {
     fprintf(stderr, "Warning: Can not lock memory.\n");
   }
+#endif
 
   initAll ();
 
@@ -830,14 +873,17 @@ int main (int argc, char * argv []) {
   }
 #endif
 
-  setMIDINoteShift (inst.midicfg, 0);
-
   setDrawBars (&inst, 0, presetSelect);
 #if 0 // initial values are assigned in tonegen.c initToneGenerator()
   setDrawBars (&inst, 1, presetSelect); /* 838 000 000 */
   setDrawBars (&inst, 2, presetSelect); /* 86 - */
 #endif
 
+  const int pgm_off = inst.progs->MIDIControllerPgmOffset;
+  if (loadProgram >= 0 && loadProgram >= pgm_off) {
+    installProgram(&inst, (loadProgram - pgm_off));
+  }
+
 #ifndef _WIN32
   signal (SIGHUP, catchsig);
   signal (SIGINT, catchsig);
@@ -850,7 +896,12 @@ int main (int argc, char * argv []) {
   fprintf(stderr,"All systems go. press CTRL-C, or send SIGINT or SIGHUP to terminate\n");
 
   while (j_client)
-    sleep (1); /* jack callback is doing this the work now */
+  /* jack callback is doing this the work now */
+#ifdef _WIN32
+    Sleep (1000);
+#else
+    sleep (1);
+#endif
 
   /* shutdown and cleanup */
 #ifdef HAVE_ASEQ
@@ -872,7 +923,9 @@ int main (int argc, char * argv []) {
     free(jack_port[i]);
 
   freeAll();
+#ifndef _WIN32
   munlockall();
+#endif
 
   fprintf(stderr, "bye\n");
   return 0;
