@@ -1,7 +1,7 @@
 /* setBfree - DSP tonewheel organ
  *
  * Copyright (C) 2003-2004 Fredrik Kilander <fk@dsv.su.se>
- * Copyright (C) 2008-2012 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2008-2015 Robin Gareus <robin@gareus.org>
  * Copyright (C) 2012 Will Panther <pantherb@setbfree.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,13 +29,14 @@
  *
  * Manager for program change.
  */
+#ifndef CONFIGDOCONLY
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/ioctl.h>
 #include <fcntl.h>
 #include <ctype.h>
 #include <string.h>
+#include <strings.h>
 #include <math.h>
 #include <sys/time.h>
 #include <unistd.h>
@@ -126,7 +127,7 @@ enum propertyId {
 };
 
 typedef struct _symbolmap {
-  char * propertyName;
+  const char * propertyName;
   int property;
 } SymbolMap;
 
@@ -173,7 +174,7 @@ static const SymbolMap propertySymbols [] = {
 /**
  * Look up the property string and return the internal property value.
  */
-static int getPropertyIndex (char * sym) {
+static int getPropertyIndex (const char * sym) {
   int i;
   for (i = 0; propertySymbols[i].propertyName != NULL; i++) {
     if (!strcasecmp (propertySymbols[i].propertyName, sym)) {
@@ -190,10 +191,10 @@ static int getPropertyIndex (char * sym) {
  * Returns the given return code, so that it may be called in a return
  * statement from within a parsing function.
  */
-static int stateMessage (char * fileName,
-			 int lineNumber,
-			 char * msg,
-			 int code) {
+static int stateMessage (const char * fileName,
+			 const int lineNumber,
+			 const char * msg,
+			 const int code) {
   fprintf (stderr, "%s in file %s on line %d\n", msg, fileName, lineNumber);
   return code;
 }
@@ -205,14 +206,14 @@ static int stateMessage (char * fileName,
  * @param lineNumber The linenumber in the input file.
  * @param fileName   The name of the current input file.
  */
-static int parseDrawbarRegistration (char * drw,
+static int parseDrawbarRegistration (const char * drw,
 				     unsigned int bar[],
-				     int    lineNumber,
-				     char * fileName) {
+				     const int    lineNumber,
+				     const char * fileName) {
 
   char msg[MESSAGEBUFFERSIZE];
   int bus = 0;
-  char * t = drw;
+  const char * t = drw;
 
   while (bus < 9) {
     if (*t == '\0') {
@@ -243,7 +244,7 @@ static int parseDrawbarRegistration (char * drw,
 /**
  * Return TRUE if the supplied string can be interpreted as an enabling arg.
  */
-static int isAffirmative (char * value) {
+static int isAffirmative (const char * value) {
   int n;
   if (!strcasecmp (value, "on")) return TRUE;
   if (!strcasecmp (value, "yes")) return TRUE;
@@ -258,7 +259,7 @@ static int isAffirmative (char * value) {
 /**
  * Return TRUE if the supplied string can be interpreted as a disabling arg.
  */
-static int isNegatory (char * value) {
+static int isNegatory (const char * value) {
   int n;
   if (!strcasecmp (value, "off")) return TRUE;
   if (!strcasecmp (value, "no")) return TRUE;
@@ -275,7 +276,7 @@ static int isNegatory (char * value) {
  * This function parses a transpose argument. It expects an integer in the
  * range -127 .. 127. It is a helper function to bindToProgram () below.
  */
-static int parseTranspose (char * val, int * vp, char * msg) {
+static int parseTranspose (const char * val, int * vp, char * msg) {
   if (sscanf (val, "%d", vp) == 0) {
     sprintf (msg, "Transpose: integer expected : '%s'", val);
     return -1;
@@ -292,11 +293,11 @@ static int parseTranspose (char * val, int * vp, char * msg) {
  * Return: 0 OK, non-zero error.
  */
 int bindToProgram (void * pp,
-		   char * fileName,
-		   int    lineNumber,
-		   int    pgmnr,
-		   char * sym,
-		   char * val)
+		   const char * fileName,
+		   const int    lineNumber,
+		   const int    pgmnr,
+		   const char * sym,
+		   const char * val)
 {
   struct b_programme *p = (struct b_programme *)pp;
   int prop;
@@ -866,15 +867,6 @@ int pgmConfig (struct b_programme *p, ConfigContext * cfg) {
 }
 #endif
 
-static const ConfigDoc doc[] = {
-  {"pgm.controller.offset", CFG_INT, "1", "Compensate for MIDI controllers that number the programs from 1 to 128. Internally we use 0-127, as does MIDI. range: [0,1]"},
-  {NULL}
-};
-
-const ConfigDoc *pgmDoc () {
-  return doc;
-}
-
 
 #define MAXROWS 18
 #define MAXCOLS  4
@@ -1151,14 +1143,23 @@ void writeProgramm(int pgmNr, Programme *p, const char *sep, FILE * fp) {
     format_drawbars(p->drawbars, tmp);
     fprintf(fp, "%s, drawbarsupper=\"%s\"", sep, tmp);
   }
+  else if ((p->flags[0] & FL_DRAWBR) && (p->flags[0] & FL_DRWRND)) {
+    fprintf(fp, "%s, drawbarsupper=random", sep);
+  }
   if ((p->flags[0] & FL_LOWDRW) && !(p->flags[0] & FL_DRWRND)) {
     format_drawbars(p->lowerDrawbars, tmp);
     fprintf(fp, "%s, drawbarslower=\"%s\"", sep, tmp);
   }
+  else if ((p->flags[0] & FL_LOWDRW) && (p->flags[0] & FL_DRWRND)) {
+    fprintf(fp, "%s, drawbarslower=random", sep);
+  }
   if ((p->flags[0] & FL_PDLDRW) && !(p->flags[0] & FL_DRWRND)) {
     format_drawbars(p->pedalDrawbars, tmp);
     fprintf(fp, "%s, drawbarspedals=\"%s\"", sep, tmp);
   }
+  else if ((p->flags[0] & FL_PDLDRW) && (p->flags[0] & FL_DRWRND)) {
+    fprintf(fp, "%s, drawbarspedals=random", sep);
+  }
   if (p->flags[0] & FL_SCANNR) {
     int knob = ((p->scanner & 0xf) << 1) - ((p->scanner & CHO_) ? 1 : 2);
     fprintf(fp, "%s, vibrato=", sep);
@@ -1290,4 +1291,20 @@ int main (int argc, char **argv) {
   return 0;
 }
 #endif
+
+#else
+# include "cfgParser.h"
+#endif // CONFIGDOCONLY
+
+static const ConfigDoc doc[] = {
+  {"pgm.controller.offset", CFG_INT, "1", "Compensate for MIDI controllers that number the programs from 1 to 128. Internally we use 0-127, as does MIDI. range: [0,1]", INCOMPLETE_DOC},
+  DOC_SENTINEL
+};
+
+const ConfigDoc *pgmDoc () {
+  return doc;
+}
+
+
+
 /* vi:set ts=8 sts=2 sw=2: */
